unit RakGlobalObjectID;
{///////////////////////////////////////////////////////////////////////////////
// Beschreibung : Die Unit RakGlobalObjectID stellt das globale Objekt
                  RakGlobalObjectManager zur Verfgung.
                  Mit RakGlobalObjectManager kann man Objekten global
                  eindeutige Namen geben. Diese Namen sollte man anstelle
                  von Referenzen in Dateien speichern, da es groer Zufall
                  wre, wenn nach dem Laden aus einer Datei ein Objekt
                  wieder die selbe Referenz htte wie vorher. Den globalen Namen
                  kann man nach dem Laden aber in der Regel problemlos wieder
                  herstellen.
// erstellt am  : ?.?.2001
////////////////////////////////////////////////////////////////////////////////
// nderungen:
///////////////////////////////////////////////////////////////////////////////}
// Delphi 5 Implementation
//
// Copyright (c) 2000 by Richard Kaspar
// ("http://www.kasparsoft.de")
//
// Definitions:
// - "Package" refers to the collection of files distributed by
//   the Copyright Holder, and derivatives of that collection of
//   files created through textual modification.
// - "Standard Version" refers to such a Package if it has not
//   been modified, or has been modified in accordance with the
//   wishes of the Copyright Holder.
// - "Copyright Holder" is whoever is name in the copyright or
//   copyrights for the package.
// - "You" is you, if you're thinking about copying or distributing
//   this Package.
//
// Permission is hereby granted, free of charge, to any person
// obtaining a copy of this software and associated documentation
// files (the "Package"), to deal in the Package without restriction,
// including without limitation the rights to use, copy, modify,
// merge, publish, distribute, sublicense, and/or sell copies of the
// Package, and to permit persons to whom the Package is furnished
// to do so, subject to the following conditions:
//
// The above copyright notice and this permission notice shall be
// included in all copies or substantial portions of the Package.
//
// You may modify your copy of this Package in any way, provided
// that you insert a prominent notice in each changed file stating
// how and when you changed a file, and provided that you do at
// least one of the following:
//
// a) allow the Copyright Holder to include your modifications in
// the Standard Version of the Package.
//
// b) use the modified Package only within your corporation or
// organization.
//
// c) rename any non standard executables, units, and classes so
// the names do not conflict with standard executables, units, and
// classes, and provide a separate manual page that clearly documents
// how it differs from the standard version.
//
// d) make other distribution arrangements with the Copyright Holder.
//
// The name of the Copyright Holder may not be used to endorse or
// promote products derived from this Package without specific prior
// written permission.
//
// THE PACKAGE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND,
// EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF
// MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT.
// IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY
// CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT,
// TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE
// PACKAGE OR THE USE OR OTHER DEALINGS IN THE PACKAGE.
{$I RAK.INC}

interface

uses SysUtils, Classes, SyncObjs;

type
{////////////////////////////////////////////////////////////////////////////////
// Klasse       : TRakGlobalObjectManager
// Beschreibung : TRakGlobalObjectManager kann fr Objekte eindeutige Namen
                  erzeugen. ber den Namen kann dann jederzeit wieder die
                  Referenz auf das Objekt ermittelt werden. Dies klappt auch
                  dann, wenn das Objekt zwischenzeitlich gelscht war.
                  Damit das Ganze aber sauber funktioniert, mssen sich
                  Objekte in ihrem Konstruktor RakGlobalObjectManager
                  anmelden (Methode AddObject) und im Destruktor wieder
                  abmelden (Methode DeleteObject). Wenn ein Objekt wiederhergestellt
                  wurde, weil es z.B. aus einer Datei geladen wurde, dann muss
                  so bald wie mglich, der alte Name mit der neuen Referenz
                  verbunden werden (Methode ChangeIdOf).
                  Wenn Programme sich beenden, dann sollten sie den Wert des
                  Properties GlobalId abspeichern, z.B. in der Registry. Beim
                  Start des Programms sollte man den Wert wieder laden und dem Property
                  wieder zuweisen. Dieses Vorgehen stellt sicher, dass selbst
                  wenn Objekte aus verschiedenen Dateien zusammen geladen werden
                  es zu keinen Konflikten mit den Objektnamen kommt.
// erstellt am  : ?.?.2001
////////////////////////////////////////////////////////////////////////////////
// nderungen:
  23.3.2002 Neues Property Objects
  23.3.2002 In die ObjId wird jetzt noch ein Zeitstempel mit aufgenommen, damit
            das Risiko eines Namenskonflikts noch weiter verringert wird.
            Es besteht aber immer noch die Gefahr dass Namen kollidieren.
            Ich wei noch nicht, wie ich eine global eindeutige ID erzeugen kann.
////////////////////////////////////////////////////////////////////////////////}
  TRakGlobalObjectManager = class
  private
    FGlobalId: Int64;
    FObjList: TStringList;
    FAddClassnameToObjId: boolean;
    FSyncObj: TCriticalSection;

    // nicht synchronisierte Methoden
    function IndexOf(Obj: TObject): integer;
    function GetCount: integer;
    function GetObjIds(Idx: integer): string;

    function NoSyncGetObjectOf(ObjId: string): TObject;
    function NoSyncGetIdOf(Obj: TObject): string;

    // synchronisierte Methoden
    procedure SetGlobalId(const Value: Int64);
    function GetGlobalId: Int64;
    function GetObjects(Idx: integer): TObject;
  public
    constructor Create;
    destructor Destroy; override;

    // AddObject verbindet "Obj" mit einen eindeutigen Namen.
    // Der eindeutige Name wird "ObjId" sein, falls dieser noch nicht vergeben ist
    // Der eindeutige Name, mit der ab jetzt mit dem Objekt verbunden ist, wird zurckgegeben
    function AddObject(Obj: TObject; ObjId: string): string;
    // DeleteObject lscht ein Objekt aus der globalen Objektverwaltung.
    // Der eindeutige Name des Objektes knnte jetzt rein theoretisch
    // fr ein neues Objekt verwendet werden. Dies sollte man allerdings nicht
    // machen, da es sonst zu Konflikten mit den Namen kommen knnte,
    // wenn das Objekt wieder neu erzeugt wird.
    procedure DeleteObject(ObjId: string); overload;
    procedure DeleteObject(Obj: TObject); overload;
    // GetObjectOf ermittelt die Referenz eines Objektes anhand seines eindeutigen Namens
    function GetObjectOf(ObjId: string): TObject;
    // GetIdOf gibt den eindeutigen Namen eines Objektes zurck
    function GetIdOf(Obj: TObject): string;
    // Mit ChangeIdOf kann man den eindeutigen Namen eines Objektes auf einen bestimmten
    // Wert setzen. Dies klappt aber nur, wenn der neuen Name nicht bereits an ein
    // anderes Objekt vergeben ist
    function ChangeIdOf(Obj: TObject; ObjId: string): boolean;

    // Count: Die Anzahl der verwalteten Objekte
    property Count: integer read GetCount;
    // ObjIds: Die eindeutigen Namen der verwalteten Objekte
    property ObjIds[Idx: integer]: string read GetObjIds; default;
    // Objects: Die verwalteten Objekte
    property Objects[Idx: integer]: TObject read GetObjects;
    // AddClassnameToObjId legt fest, ob der Name der Klasse in den Objektnamen einflieen soll
    // Dies kann es einem beim Debuggen erleichtern zu erkennen, mit welchem Objekt man es zu tun hat
    property AddClassnameToObjId: boolean read FAddClassnameToObjId write FAddClassnameToObjId;
    // GlobalId sollte man beim Programmende wegspeichern und beim Programmstart laden und neu setzen
    // Damit wird die Wahrscheinlichkeit von Namenskollisionen verringert
    property GlobalId: Int64 read GetGlobalId write SetGlobalId;
  end;

var
  RakGlobalObjectManager: TRakGlobalObjectManager;

implementation

{ TRakGlobalObjectManager }

function TRakGlobalObjectManager.AddObject(Obj: TObject;
  ObjId: string): string;
var
  Counter: integer;
  Id: integer;
begin
  Result := '';
  FSyncObj.Enter;
  try
    if (Obj <> nil) and (GetIdOf(Obj) = '') then
    begin
      Result := ObjId;
      if (Result = '') or (GetObjectOf(Result) <> nil) then
      begin
        Counter := 1;
        inc(FGlobalId);
        Id := FGlobalId;
        if AddClassnameToObjId then
          ObjId := ObjId + IntToStr(Id) + Obj.ClassName
        else
          ObjId := ObjId + IntToStr(Id);

        ObjId := ObjId + FloatToStr(Now);

        Result := ObjId;
        while GetObjectOf(Result) <> nil do
        begin
          inc(Counter);
          Result := ObjId + '_' + IntToStr(Counter);
        end;
      end;

      FObjList.AddObject(Result, Obj);
    end;
  finally
    FSyncObj.Leave;
  end;
end;

function TRakGlobalObjectManager.ChangeIdOf(Obj: TObject; ObjId: string): boolean;
var
  Idx: integer;
begin
  Result := false;
  if ObjId <> '' then
  begin
    FSyncObj.Enter;
    try
      Idx := IndexOf(Obj);
      if Idx >= 0 then
      begin
        if CompareText(FObjList[Idx], ObjId) = 0 then
          Result := true
        else if NoSyncGetObjectOf(ObjId) = nil then
        begin
          FObjList.Sorted := false;
          FObjList[Idx] := ObjId;
          FObjList.Sorted := true;
          Result := true;
        end;
      end;
    finally
      FSyncObj.Leave;
    end;
  end;
end;

constructor TRakGlobalObjectManager.Create;
begin
  if RakGlobalObjectManager <> nil then
    raise Exception.Create('TRakGlobalObjectManager is a Singleton');

  inherited;
  FSyncObj := TCriticalSection.Create;
  FObjList := TStringList.Create;
  FObjList.Sorted := true;
  AddClassnameToObjId := true;
end;

procedure TRakGlobalObjectManager.DeleteObject(Obj: TObject);
var
  Idx: integer;
begin
  FSyncObj.Enter;
  try
    Idx := IndexOf(Obj);
    if Idx >= 0 then
      FObjList.Delete(Idx);
  finally
    FSyncObj.Leave;
  end;
end;

procedure TRakGlobalObjectManager.DeleteObject(ObjId: string);
var
  Idx: integer;
begin
  FSyncObj.Enter;
  try
    Idx := FObjList.IndexOf(ObjId);
    if Idx >= 0 then
      FObjList.Delete(Idx);
  finally
    FSyncObj.Leave;
  end;
end;

destructor TRakGlobalObjectManager.Destroy;
begin
  FObjList.Free;
  FSyncObj.Free;

  inherited;
end;

function TRakGlobalObjectManager.GetCount: integer;
begin
  Result := FObjList.Count;
end;

function TRakGlobalObjectManager.GetGlobalId: Int64;
begin
  FSyncObj.Enter;
  try
    Result := FGlobalId;
  finally
    FSyncObj.Leave;
  end;
end;

function TRakGlobalObjectManager.GetIdOf(Obj: TObject): string;
begin
  FSyncObj.Enter;
  try
    Result := NoSyncGetIdOf(Obj);
  finally
    FSyncObj.Leave;
  end;
end;

function TRakGlobalObjectManager.GetObjectOf(ObjId: string): TObject;
begin
  FSyncObj.Enter;
  try
    Result := NoSyncGetObjectOf(ObjId);
  finally
    FSyncObj.Leave;
  end;
end;

function TRakGlobalObjectManager.GetObjects(Idx: integer): TObject;
begin
  Result := FObjList.Objects[Idx];
end;

function TRakGlobalObjectManager.GetObjIds(Idx: integer): string;
begin
  Result := FObjList[Idx];
end;

function TRakGlobalObjectManager.IndexOf(Obj: TObject): integer;
var
  i: integer;
begin
  Result := -1;
  for i := 0 to FObjList.Count - 1 do
  begin
    if FObjList.Objects[i] = Obj then
    begin
      Result := i;
      break;
    end;
  end;
end;

function TRakGlobalObjectManager.NoSyncGetIdOf(Obj: TObject): string;
var
  Idx: integer;
begin
  Result := '';

  Idx := IndexOf(Obj);
  if Idx >= 0 then
    Result := FObjList[Idx];
end;

function TRakGlobalObjectManager.NoSyncGetObjectOf(ObjId: string): TObject;
var
  Idx: integer;
begin
  Result := nil;

  Idx := FObjList.IndexOf(ObjId);
  if Idx >= 0 then
    Result := TObject(FObjList.Objects[Idx]);
end;

procedure TRakGlobalObjectManager.SetGlobalId(const Value: Int64);
begin
  FSyncObj.Enter;
  try
    FGlobalId := Value;
  finally
    FSyncObj.Leave;
  end;
end;

initialization
  RakGlobalObjectManager := nil;
  RakGlobalObjectManager := TRakGlobalObjectManager.Create;

finalization
  RakGlobalObjectManager.Free;
  RakGlobalObjectManager := nil;

end.
