unit BitStream;

interface

uses DBTables, DB;

type
	TBitArea = 0..(high(integer)-1);
  TBitSize = 0..(high(TBitArea)+1);
	TArraySize = 0..(high(TBitArea) div 8 + 1);
	TArrayArea = 0..(high(TArraySize)+1);
	TBitStreamArray = array[TArrayArea] of byte;
  PBitStreamArray = ^TBitStreamArray;

  TBitStream = class
  private
  	FBytes: PBitStreamArray;
    FBytesCount: TArraySize;
    FBitPointer: integer;
    FBitsCount: TBitSize;
    FResetByRedim: boolean;
    FTrueNo: integer;
    FEOF: boolean;
    FVarBytes: Variant;

    function GetBits(Index: TBitArea): boolean;
    procedure SetBits(Index: TBitArea; Value: boolean);
    procedure SetBitsCount(Value: TBitSize);
    function GetBytes(Index: TArrayArea): byte;
    procedure SetBytes(Index: TArrayArea; Value: Byte);
    procedure SetBytesCount(Value: TArraySize);
  public
  	constructor Create;
    destructor Destroy; override;

    procedure Assign(Src: TBitStream);
    procedure BitAnd(Src: TBitStream);
    procedure BitOr(Src: TBitStream);
    procedure BitXor(Src: TBitStream);
    function CountTrue: integer;
    procedure Load(Field: TBlobField);
    procedure NegiereBits;
    procedure Save(Field: TBlobField);

    procedure SetAllBits(Checked: boolean);

    function First: integer;
    function Next: integer;
    function FindTrueNo(TrueNo: integer): integer;

    property Bits[Index: TBitArea]: boolean read GetBits write SetBits; default;
    property BitsCount: TBitSize read FBitsCount write SetBitsCount;
    property Bytes[Index: TArrayArea]: byte read GetBytes write SetBytes;
    property BytesCount: TArraySize read FBytesCount write SetBytesCount;

    property ResetByRedim: boolean read FResetByRedim write FResetByRedim;

    property TrueNo: integer read FTrueNo;
    property EOF: boolean read FEOF;
  end;


implementation

constructor TBitStream.Create;
begin
	inherited Create;

  FResetByRedim := true;
  FBytesCount := 0;
  FBitsCount := 0;
  FVarBytes := VarArrayCreate([0,0], varByte);
  FBytes := VarArrayLock(FVarBytes);
end;

destructor TBitStream.Destroy;
begin
  inherited Destroy;
end;

procedure TBitStream.Assign(Src: TBitStream);
var
	i: integer;
begin
	FTrueNo := -1;
	for i := 0 to BytesCount - 1 do
  begin
  	FBytes[i] := Src.FBytes[i];
  end;
end;

procedure TBitStream.BitAnd(Src: TBitStream);
var
	i: integer;
begin
	FTrueNo := -1;
	for i := 0 to BytesCount - 1 do
  begin
  	FBytes[i] := FBytes[i] and Src.FBytes[i];
  end;
end;

procedure TBitStream.BitOr(Src: TBitStream);
var
	i: integer;
begin
	FTrueNo := -1;
	for i := 0 to BytesCount - 1 do
  begin
  	FBytes[i] := FBytes[i] or Src.FBytes[i];
  end;
end;

procedure TBitStream.NegiereBits;
var
	i: integer;
begin
	FTrueNo := -1;
	for i := 0 to BytesCount - 1 do
  begin
  	FBytes[i] := FBytes[i] xor 255;
  end;
end;

procedure TBitStream.BitXor(Src: TBitStream);
var
	i: integer;
begin
	FTrueNo := -1;
	for i := 0 to BytesCount - 1 do
  begin
  	FBytes[i] := FBytes[i] xor Src.FBytes[i];
  end;
end;

function TBitStream.CountTrue: integer;
var
	i: integer;
begin
	Result := 0;
	for i := 0 to BitsCount - 1 do
  begin
  	if Bits[i] then
    	inc(Result);
  end;
end;

procedure TBitStream.Load(Field: TBlobField);
var
  BlobStream: TBlobStream;
begin
  BlobStream := TBlobStream.Create(Field, bmread);
  try
    BlobStream.Read(FBytes^, BytesCount);
  finally
    BlobStream.Destroy;
  end;
end;

procedure TBitStream.Save(Field: TBlobField);
var
  BlobStream: TBlobStream;
begin
  BlobStream := TBlobStream.Create(Field, bmwrite);
  try
    BlobStream.Write(FBytes^, BytesCount);
  finally
    BlobStream.Destroy;
  end;
end;

procedure TBitStream.SetAllBits(Checked: boolean);
var
	i: integer;
  Wert: byte;
begin
	if Checked then
  begin
  	Wert := 255;
    FEOF := false;
    FBitPointer := 0;
    FTrueNo := 0;
  end
  else
  begin
  	Wert := 0;
    FEOF := true;
    FBitPointer := -1;
    FTrueNo := -1;
  end;

	for i := 0 to FBytesCount do
    FBytes[i] := Wert;
end;

function TBitStream.GetBits(Index: TBitArea): boolean;
begin
	Result := ((FBytes[Index div 8] shr (Index mod 8)) and 1) = 1;
end;

procedure TBitStream.SetBits(Index: TBitArea; Value: boolean);
var
	Id: integer;
begin
	if Bits[Index] <> Value then
  begin
    Id := Index div 8;
    if Value then
      FBytes[Id] := FBytes[Id] or (1 shl (Index mod 8))
    else
      FBytes[Id] := FBytes[Id] and ((1 shl (Index mod 8)) xor 255);

    if FBitPointer >= Index then
	  	FTrueNo := -1;
  end;
end;

procedure TBitStream.SetBitsCount(Value: TBitSize);
begin
	if FBitsCount <> Value then
  	FTrueNo := -1;

	FBitsCount := Value;
  BytesCount := FBitsCount div 8 + 1;
end;

function TBitStream.GetBytes(Index: TArrayArea): Byte;
begin
	Result := FBytes[Index];
end;

procedure TBitStream.SetBytes(Index: TArrayArea; Value: byte);
begin
	FBytes[Index] := Value;
end;

procedure TBitStream.SetBytesCount(Value: TArraySize);
begin
	VarArrayUnlock(FVarBytes);
  VarArrayRedim(FVarBytes, Value);
	FBytes := VarArrayLock(FVarBytes);
  FBytesCount := Value;
  if BytesCount <> FBitsCount div 8 + 1 then
  	FBitsCount := BytesCount * 8;
  if ResetByRedim then
	  SetAllBits(false);
end;

function TBitStream.First: integer;
begin
	FTrueNo := -1;
  FBitPointer := -1;
	Result := FindTrueNo(0);
end;

function TBitStream.Next: integer;
begin
	Result := FindTrueNo(TrueNo + 1);
end;

function TBitStream.FindTrueNo(TrueNo: integer): integer;
var
	i: integer;
  lCounter: integer;
begin
	FEOF := true;

	if FTrueNo = TrueNo then
  begin
  	Result := FBitPointer;
    FEOF := false;
  end
  else
  begin
    if (FTrueNo >= 0) and (FBitPointer >= 0) then
    begin
      lCounter := FTrueNo;
      i := FBitPointer;
    end
    else
    begin
    	lCounter := -1;
      i := -1;
    end;
    Result := -1;
    if lCounter < TrueNo then
    begin
    	inc(i);
      while (Result = -1) and (i < BitsCount) do
      begin
        if Bits[i] then
        begin
          inc(lCounter);
          if lCounter = TrueNo then
          begin
            FTrueNo := TrueNo;
            FEOF := false;
            FBitPointer := i;
            Result := i;
          end;
        end;
        inc(i);
      end;
    end
    else if lCounter > TrueNo then
    begin
    	dec(i);
      while (Result = -1) and (i >= 0) do
      begin
        if Bits[i] then
        begin
          dec(lCounter);
          if lCounter = TrueNo then
          begin
            FTrueNo := TrueNo;
            FEOF := false;
            FBitPointer := i;
            Result := i;
          end;
        end;
        dec(i);
      end;
    end;
  end;
end;



end.
