unit MultiEvent;
{///////////////////////////////////////////////////////////////////////////////
// Beschreibung : Die unit MultiEvent stellt Klassen zur Verfgung, mit denen
                  Ereignisse ausgelst werden knnen, von denen nicht ein,
                  sondern beliebig viele Objekte etwas mitbekommen knnen.
                  Bei den normalen Delphi-Ereignissen kann sich immer nur
                  genau ein Objekt einhngen. Es kann aber durchaus sein,
                  dass mehrere Objekte auf ein Ereignis reagieren wollen.
                  Wenn z.B. mehrere Objekte eine Referenz auf ein bestimmtes
                  Objekt haben, dann mssen alle Objekte mitbekommen, wenn
                  sich das Objekt lscht, damit die Referenzen bereinigt
                  werden knnen.
                  Die Klassen TRakEventObject und TRakEventVerwalter
                  stellen das allgemeine Gerst zur Verfgung. Aber erst davon
                  abgeleitete Klassen knnen tatschlich bestimmte Events
                  auslsen. Ein Beispiel dafr bilden die Klassen
                  TRakNotifyEvent und TRakNotifyEventVerwalter.
// erstellt am  : 26.04.2001
////////////////////////////////////////////////////////////////////////////////
// nderungen:
///////////////////////////////////////////////////////////////////////////////}
// Delphi 5 Implementation
//
// Copyright (c) 2000 by Richard Kaspar
// ("http://www.kasparsoft.de")
//
// Definitions:
// - "Package" refers to the collection of files distributed by
//   the Copyright Holder, and derivatives of that collection of
//   files created through textual modification.
// - "Standard Version" refers to such a Package if it has not
//   been modified, or has been modified in accordance with the
//   wishes of the Copyright Holder.
// - "Copyright Holder" is whoever is name in the copyright or
//   copyrights for the package.
// - "You" is you, if you're thinking about copying or distributing
//   this Package.
//
// Permission is hereby granted, free of charge, to any person
// obtaining a copy of this software and associated documentation
// files (the "Package"), to deal in the Package without restriction,
// including without limitation the rights to use, copy, modify,
// merge, publish, distribute, sublicense, and/or sell copies of the
// Package, and to permit persons to whom the Package is furnished
// to do so, subject to the following conditions:
//
// The above copyright notice and this permission notice shall be
// included in all copies or substantial portions of the Package.
//
// You may modify your copy of this Package in any way, provided
// that you insert a prominent notice in each changed file stating
// how and when you changed a file, and provided that you do at
// least one of the following:
//
// a) allow the Copyright Holder to include your modifications in
// the Standard Version of the Package.
//
// b) use the modified Package only within your corporation or
// organization.
//
// c) rename any non standard executables, units, and classes so
// the names do not conflict with standard executables, units, and
// classes, and provide a separate manual page that clearly documents
// how it differs from the standard version.
//
// d) make other distribution arrangements with the Copyright Holder.
//
// The name of the Copyright Holder may not be used to endorse or
// promote products derived from this Package without specific prior
// written permission.
//
// THE PACKAGE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND,
// EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF
// MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT.
// IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY
// CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT,
// TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE
// PACKAGE OR THE USE OR OTHER DEALINGS IN THE PACKAGE.
{$I RAK.INC}

interface

uses classes,
  RakCollection;

type

  TRakEventVerwalter = class;
  TRakEventObjectClass = class of TRakEventObject;

  {////////////////////////////////////////////////////////////////////////////////
  // Klasse       : TRakEventObject
  // Beschreibung : Dies ist die Basisklasse fr alle Objekte, die Ereignisse
  //                empfangen knnen.
                    Ein TRakEventObject kann mit mehreren TRakEventVerwaltern
                    verbunden sein.
  //                Siehe auch Klassen TRakEventVerwalter und TRakNotifyEvent
  // erstellt am  : 26.04.2001
  ////////////////////////////////////////////////////////////////////////////////
  // nderungen:
      21.9.2001: Ereignis OnGetrennt eingefhrt
      27.11.2001: Neue Methode DisconnectAll
                  Neue Properties Count und Verwalter
  ////////////////////////////////////////////////////////////////////////////////}
  TRakEventObject = class
  private
    FSender: TList;
    FOnGetrennt: TNotifyEvent;
    function GetCount: integer;
    function GetVerwalter(Idx: integer): TRakEventVerwalter;
  protected
    procedure EventVerwalterGetrennt(EventVerwalter: TRakEventVerwalter); virtual;
    property Sender: TList read FSender;
  public
    constructor Create; virtual;
    destructor Destroy; override;

    function ListensTo(Verwalter: TRakEventVerwalter): boolean; overload; virtual;
    procedure DisconnectAll; virtual;

    property Count: integer read GetCount;
    property Verwalter[Idx: integer]: TRakEventVerwalter read GetVerwalter; default;

    property OnGetrennt: TNotifyEvent read FOnGetrennt write FOnGetrennt;
  end;

  ////////////////////////////////////////////////////////////////////////////////
  // Klasse       : TRakEventVerwalter
  // Beschreibung : Stellt den allgemeinen Mechanismus zur Verfgung, mit dem
  //                ein Ereignis an mehrere Objekte verteilt werden kann.
  //                Die Klasse hat zwar keine abstrakten Methoden, aber eine
  //                Instanz davon wre fr nichts zu gebrauchen. Erst davon
  //                abgeleitete Klassen bestimmen welches Ereignis
  //                durch sie ausgelst werden kann, siehe z.B. Klasse
  //                TRakNotifyEventVerwalter.
  //                Die EventObjekte (TRakNotifyEvent) gehren nicht dem
  //                Verwalter. Er zestrt sie also nicht, wenn er sich selbst
  //                zerstrt.
  // erstellt am  : 26.04.2001
  ////////////////////////////////////////////////////////////////////////////////
  // nderungen:
  //  27.11.2001: Neue Methode Delete
  ////////////////////////////////////////////////////////////////////////////////
  TRakEventVerwalter = class
  private
    FEventList: TList;
  protected
    FDestroying: boolean;
    FEventObjectClass: TRakEventObjectClass;

    procedure DestroyingEventObject(Obj: TRakEventObject); virtual;

    property EventList: TList read FEventList;
  public
    constructor Create(EventObjectClass: TRakEventObjectClass);
    destructor Destroy; override;

    function Add: TRakEventObject; overload; virtual;
    procedure Add(EventObject: TRakEventObject); overload; virtual;
    procedure Delete(EventObject: TRakEventObject); overload; virtual;
  end;

  ////////////////////////////////////////////////////////////////////////////////
  // Klasse       : TRakNotifyEvent
  // Beschreibung : siehe TRakNotifyEventVerwalter
  // erstellt am  : 26.04.2001
  ////////////////////////////////////////////////////////////////////////////////
  // nderungen:
  ////////////////////////////////////////////////////////////////////////////////
  TRakNotifyEvent = class(TRakEventObject)
  private
    FEvent: TNotifyEvent;
  public
    procedure FireEvent(Sender: TObject); virtual;

    property Event: TNotifyEvent read FEvent write FEvent;
  end;

  ////////////////////////////////////////////////////////////////////////////////
  // Klasse       : TRakNotifyEventVerwalter
  // Beschreibung : Diese Klasse kann verwendet werden, wenn ein einfaches
  //                Ereignis ausgelst werden soll, auf das mehrere andere
  //                Objekte reagieren sollen.
  //                Die Objekte, die auf das Ereignis reagieren sollen,
  //                rufen die Methode AddEventObj auf, damit Sie ein
  //                TRakNotifyEvent-Objekt bekommen. ber das Property Event
  //                des TRakNotifyEvent-Objekts wird dann ein ganz normales
  //                Delphi-Ereignis ausgelst.
                    {  Beispiel:
                    // TKlasse1 lst ein Ereignis aus, wenn es sich zerstrt
                    TKlasse1 = class
                      ...
                      destructor Destroy; override;
                      property OnDestroy: TRakNotifyEventVerwalter read FOnDestroy;
                    end;
                    ...
                    // TKlasse2 kann sich bei einer Instanz von TKlasse1
                    // einhngen und bekommt mit, wenn sich diese lscht
                    TKlasse2 = class
                      ...
                      FEventObj: TRakNotifyEvent;
                      destructor Destroy; override;
                      procedure ListenTo(Obj: TKlasse1);
                      procedure ObjDestroyed(Sender: TObject);
                    end;
                    ...
                    destructor TKlasse1.Destroy;
                    begin
                      OnDestroy.FireEvent(self); // Das Ereignis auslsen
                      OnDestroy.Free;
                      inherited;
                    end;
                    ...
                    destructor TKlasse2.Destroy;
                    begin
                      FEventObj.Free;// Das EventObject gehrt TKlasse2, deswegen
                        // muss sie es auch zerstren.
                        // Man muss sich nicht darum kmmern, dass die
                        // Verbindung zwischen FEventObj und dem zugehrigen
                        // TKlasse1.OnDestroy sauber getrennt wird. Das machen
                        // die beiden unter sich aus
                      inherited;
                    end;

                    procedure TKlasse2.ListenTo(Obj: TKlasse1);
                    begin
                      FEventObj.Free;//Ein evtl. altes EventObj freigeben
                      FEventObj := Obj.OnDestroy.AddEventObj; // Eine neues
                        // EventObject erzeugen, das benachrichtigt wird,
                        // wenn sich Obj zerstrt
                      FEventObj.Event := ObjDestroyed; // Die Methode
                        // festlegen, die aufgerufen werden soll, wenn
                        // sich Obj zerstrt
                    end;

                    procedure TKlasse2.ObjDestroyed(Sender: TObject);
                    begin
                      ShowMessage('Das Objekt hat sich zerstrt');
                    end;
                    }
  // erstellt am  : 26.04.2001
  ////////////////////////////////////////////////////////////////////////////////
  // nderungen:
  ////////////////////////////////////////////////////////////////////////////////
  TRakNotifyEventVerwalter = class(TRakEventVerwalter)
  public
    constructor Create;

    function AddEventObj: TRakNotifyEvent;
    procedure FireEvent(Sender: TObject); virtual;
  end;

  {////////////////////////////////////////////////////////////////////////////////
  // Klasse       : TRakEventItem
  // Beschreibung : Wird von TRakEventList verwendet, um die Freigabe von
                    Event-Objekte zu vereinfachen
                    TRakEventItem ist der Besitzer von FEvent. FEvent wird
                    zerstrt, wenn die Instanz von TRakEventItem zerstrt wird.
                    Die Instanz von TRakEventItem zerstrt sich selbst,
                    wenn die Verbindung von FEvent zu seinem EventVerwalter
                    getrennt wird.
  // erstellt am  : 21.9.2001
  ////////////////////////////////////////////////////////////////////////////////
  // nderungen:
      27.11.01: Gibt sich jetzt erst frei, wenn sich das EventObjekt von all seinen
                EventVerwaltern getrennt hat.
  ////////////////////////////////////////////////////////////////////////////////}
  TRakEventItem = class(TRakCollectionItem)
  private
    FEvent: TRakEventObject;
    FDestroying: boolean;
    procedure Getrennt(Sender: TObject);
  public
    Sender: TObject;

    destructor Destroy; override;

    property Event: TRakEventObject read FEvent;
  end;

  {////////////////////////////////////////////////////////////////////////////////
  // Klasse       : TRakEventList
  // Beschreibung : TRakEventList kann von Objekten verwendet werden, die auf
                    mehrere Multievents reagieren mssen, damit die Verwaltung
                    der Event-Objekte vereinfacht wird.
                    Wenn von einem Eventverwalter ein Eventobjekt angefordert wird,
                    dann muss man sich selbst um die Freigabe des Eventobjektes kmmern.
                    Diese Aufgabe kann an TRakEventList bertragen werden (RakEventList.Add(EventObjekt))
                    Sollte die Verbindung zw. den EventObjekt und seinem EventVerwalter getrennt
                    werden, dann wird automatisch das EventObjekt freigegeben.

                    Wenn man auf die Ereignisse eines Objektes irgendwann einmal
                    nicht mehr reagieren mchte, dann kann dies durch die Methode
                    FreeSenderEvents erreicht werden. Dies funktioniert allerdings nur,
                    wenn man das EventObjekt mit dem Add aufgenommen hat, das auch
                    den Sender erwartet (RakEventList.Add(EventObjekt, SenderObjekt))

                    Beispiel:
                    TEinSender = class
                    ...
                    public
                      property OnEinEreignis: TRakNotifyEventVerwalter;
                      ...
                    end;

                    TEmpfaenger = class
                    private
                      FEvents: TRakEventList;

                      procedure SenderEreignis(Sender: TObject);
                    public
                      constructor Create;
                      destructor Destroy; override;

                      procedure HoereAuf(Sender: TEinSender);
                      procedure HoereNichtMehrAuf(Sender: TEinSender);
                    end;

                    ...

                    constructor TEmpfaenger.Create;
                    begin
                      FEvents := TRakEventList.Create;
                    end;

                    destructor TEmpfaenger.Destroy;
                    begin
                      FEvents.Free;
                      inherited;
                    end;

                    procedure TEmpfaenger.HoereAuf(Sender: TEinSender);
                    var
                      EventObj: TRakNotifyEvent;
                    begin
                      EventObj := Sender.OnEinEreignis.AddEventObj;
                      FEvents.Add(EventObj, Sender);
                      EventObj.Event := SenderEreignis;
                    end;

                    procedure HoereNichtMehrAuf(Sender: TEinSender);
                    begin
                      FEvents.FreeSenderEvents(Sender);
                    end;

                    procedure TEmpfaenger.SenderEreignis(Sender: TObject);
                    begin
                      // Auf das Ereignis reagieren
                    end;

  // erstellt am  : 21.9.2001
  ////////////////////////////////////////////////////////////////////////////////
  // nderungen:
  ////////////////////////////////////////////////////////////////////////////////}
  TRakEventList = class(TRakCollection)
  private
    function GetItems(Idx: integer): TRakEventItem;
  public
    function Add(Event: TRakEventObject): TRakEventItem; overload;
    function Add(Event: TRakEventObject; Sender: TObject): TRakEventItem; overload;

    function Find(Sender: TObject): TRakEventItem; overload;
    function FindByVerwalter(Verwalter: TRakEventVerwalter): TRakEventItem; overload;
    procedure FreeSenderEvents(Sender: TObject);

    property Items[Idx: integer]: TRakEventItem read GetItems; default;
  end;


implementation

{ TRakEventVerwalter }

////////////////////////////////////////////////////////////////////////////////
// Klasse       : TRakEventVerwalter
// Methode      : Add
// Sichtbarkeit : public
// Virtual      : ja
// Beschreibung : Nimmt ein EventObject in die Liste der Objekte auf, die auf
//                ein Ereignisse von diesem EventVerwalter warten.
// Parameter
//    in        : EventObject: Das neue EventObject
//                             EventObject muss von der Klasse (oder davon
//                             abgeleitet) sein, die durch den Aufruf
//                             des Konstruktor festgelegt wurde
//    out       :
//    var       :
//    const     :
// Rckgabe     :
// erstellt am  : 26.04.2001
////////////////////////////////////////////////////////////////////////////////
// nderungen:
//  27.11.01: Ein EventObject kann jetzt nur noch einmal aufgenommen werden
////////////////////////////////////////////////////////////////////////////////
procedure TRakEventVerwalter.Add(EventObject: TRakEventObject);
begin
  if EventObject is FEventObjectClass then
  begin
    if EventList.IndexOf(EventObject) = -1 then
    begin
      EventObject.Sender.Add(self);
      EventList.Add(EventObject);
    end;
  end;
end;

////////////////////////////////////////////////////////////////////////////////
// Klasse       : TRakEventVerwalter
// Methode      : Add
// Sichtbarkeit : public
// Virtual      : ja
// Beschreibung : Erzeugt ein neues EventObject, das auf Ereignisse von diesem
//                EventVerwalter wartet. Die Klasse des EventObjects wurde
//                beim Aufruf des Konstruktors festgelegt.
// Parameter
//    in        :
//    out       :
//    var       :
//    const     :
// Rckgabe     : TRakEventObject; Gibt die neue Instanz eines EventObjects
//                                  zurck. Der Aurufer wird zum Besitzer der
//                                  Instanz.
// erstellt am  : 26.04.2001
////////////////////////////////////////////////////////////////////////////////
// nderungen:
////////////////////////////////////////////////////////////////////////////////
function TRakEventVerwalter.Add: TRakEventObject;
begin
  Result := FEventObjectClass.Create;
  Add(Result);
end;

////////////////////////////////////////////////////////////////////////////////
// Klasse       : TRakEventVerwalter
// Konstruktor  : Create
// Sichtbarkeit : public
// Virtual      : nein
// Beschreibung :
// Parameter
//    in        : EventObjectClass: Legt die Klasse fest, von der EventObjects
//                                  sein mssen, damit man sie mit der
//                                  Methode Add aufnehmen kann.
//    out       :
//    var       :
//    const     :
// erstellt am  : 26.04.2001
////////////////////////////////////////////////////////////////////////////////
// nderungen:
////////////////////////////////////////////////////////////////////////////////
constructor TRakEventVerwalter.Create(
  EventObjectClass: TRakEventObjectClass);
begin
  inherited Create;

  FEventObjectClass := EventObjectClass;

  FEventList := TList.Create;
end;

////////////////////////////////////////////////////////////////////////////////
// Klasse       : TRakEventVerwalter
// Methode      : Delete
// Beschreibung : Nimmt das Objekt auis der Liste der zu benachrichtigenden
//                Objkte heraus
// erstellt am  : 27.11.2001
////////////////////////////////////////////////////////////////////////////////
// nderungen:
////////////////////////////////////////////////////////////////////////////////
procedure TRakEventVerwalter.Delete(EventObject: TRakEventObject);
var
  Idx: integer;
begin
  Idx := EventList.IndexOf(EventObject);
  if Idx >= 0 then
  begin
    EventList.Delete(Idx);
    EventObject.EventVerwalterGetrennt(self);
  end;
end;

////////////////////////////////////////////////////////////////////////////////
// Klasse       : TRakEventVerwalter
// Destruktor   : Destroy
// Beschreibung : Teilt allen EventObjects mit, dass dieser EventVerwalter
//                keine Ereignisse mehr auslsen wird
// erstellt am  : 26.04.2001
////////////////////////////////////////////////////////////////////////////////
// nderungen:
////////////////////////////////////////////////////////////////////////////////
destructor TRakEventVerwalter.Destroy;
var
  EventObject: TRakEventObject;
begin
  if not FDestroying then
  begin
    FDestroying := true;

    while EventList.Count > 0 do
    begin
      EventObject := TRakEventObject(EventList[EventList.Count - 1]);
      EventList.Delete(EventList.Count - 1);
      EventObject.EventVerwalterGetrennt(self);
    end;

    FEventList.Free;
    FEventList := nil;

    inherited;
  end;
end;

////////////////////////////////////////////////////////////////////////////////
// Klasse       : TRakEventVerwalter
// Methode      : DestroyingEventObject
// Sichtbarkeit : protected
// Virtual      : ja
// Beschreibung : Wird von einem EventObject aufgerufen das sich gerade zerstrt
//                Das EventObject wird aus der Liste der zu benachrichtigenden
//                Objekte herausgenommen
// Parameter
//    in        : Obj: Das sich lschende EventObject
//    out       :
//    var       :
//    const     :
// Rckgabe     :
// erstellt am  : 26.04.2001
////////////////////////////////////////////////////////////////////////////////
// nderungen:
////////////////////////////////////////////////////////////////////////////////
procedure TRakEventVerwalter.DestroyingEventObject(Obj: TRakEventObject);
begin
  if not FDestroying then
    FEventList.Extract(Obj);
end;

{ TRakEventObject }

////////////////////////////////////////////////////////////////////////////////
// Klasse       : TRakEventObject
// Konstruktor  : Create
// Sichtbarkeit : public
// Virtual      : nein
// Beschreibung :
// Parameter
//    in        :
//    out       :
//    var       :
//    const     :
// erstellt am  : 26.04.2001
////////////////////////////////////////////////////////////////////////////////
// nderungen:
////////////////////////////////////////////////////////////////////////////////
constructor TRakEventObject.Create;
begin
  FSender := TList.Create;
end;

////////////////////////////////////////////////////////////////////////////////
// Klasse       : TRakEventObject
// Destruktor   : Destroy
// Beschreibung : Teilt allen Eventverwaltern mit, dass sich das Objekt zerstrt
// erstellt am  : 26.04.2001
////////////////////////////////////////////////////////////////////////////////
// nderungen:
////////////////////////////////////////////////////////////////////////////////
destructor TRakEventObject.Destroy;
begin
  DisconnectAll;

  FSender.Free;
  FSender := nil;

  inherited;
end;

////////////////////////////////////////////////////////////////////////////////
// Klasse       : TRakEventObject
// Methode      : DisconnectAll
// Sichtbarkeit : public
// Virtual      : ja
// Beschreibung : Trennt die Verbindungen zu allen EventVerwaltern
// Parameter
//    in        :
//    out       :
//    var       :
//    const     :
// Rckgabe     :
// erstellt am  : 27.11.2001
////////////////////////////////////////////////////////////////////////////////
// nderungen:
////////////////////////////////////////////////////////////////////////////////
procedure TRakEventObject.DisconnectAll;
var
  i: integer;
begin
  for i := 0 to FSender.Count - 1 do
    TRakEventVerwalter(FSender[i]).DestroyingEventObject(self);
    
  FSender.Clear;
end;

////////////////////////////////////////////////////////////////////////////////
// Klasse       : TRakEventObject
// Methode      : EventVerwalterGetrennt
// Sichtbarkeit : protected
// Virtual      : ja
// Beschreibung : Wird von einem EventVerwalter aufgerufen, um den
//                Objekt mitzuteilen, dass der EventVerwalter von dem
//                Objekt keine Ereignisse mehr auslsen wird
// Parameter
//    in        : EventVerwalter: Der EventVerwalter, der sich von dem Objekt
//                                getrennt hat.
//    out       :
//    var       :
//    const     :
// Rckgabe     :
// erstellt am  : 26.04.2001
////////////////////////////////////////////////////////////////////////////////
// nderungen:
////////////////////////////////////////////////////////////////////////////////
procedure TRakEventObject.EventVerwalterGetrennt(
  EventVerwalter: TRakEventVerwalter);
begin
  FSender.Remove(EventVerwalter);
  if Assigned(FOnGetrennt) then
    FOnGetrennt(self);
end;

////////////////////////////////////////////////////////////////////////////////
// Klasse       : TRakEventObject
// Methode      : GetCount
// Sichtbarkeit : public
// Virtual      : nein
// Beschreibung : read-Methode des Properties Count
// Parameter
//    in        :
//    out       :
//    var       :
//    const     :
// Rckgabe     : Anzahl der Verwalter
// erstellt am  : 27.11.2001
////////////////////////////////////////////////////////////////////////////////
// nderungen:
////////////////////////////////////////////////////////////////////////////////
function TRakEventObject.GetCount: integer;
begin
  Result := FSender.Count;
end;

////////////////////////////////////////////////////////////////////////////////
// Klasse       : TRakEventObject
// Methode      : GetVerwalter
// Sichtbarkeit : public
// Virtual      : nein
// Beschreibung : read-Methode des Properties Verwalter
// Parameter
//    in        : Idx: Index des Verwalters (0 <= Idx < Count)
//    out       :
//    var       :
//    const     :
// Rckgabe     : Der entspr. Verwalter
// erstellt am  : 27.11.2001
////////////////////////////////////////////////////////////////////////////////
// nderungen:
////////////////////////////////////////////////////////////////////////////////
function TRakEventObject.GetVerwalter(Idx: integer): TRakEventVerwalter;
begin
  Result := TRakEventVerwalter(FSender[Idx]);
end;

////////////////////////////////////////////////////////////////////////////////
// Klasse       : TRakEventObject
// Methode      : ListensTo
// Sichtbarkeit : public
// Virtual      : ja
// Beschreibung : Stellt fest, ob das Objekt auf Ereignisse eines bestimmten
//                Eventverwalters wartet
// Parameter
//    in        : Verwalter: Der EventVerwalter auf den getestet werden soll
//    out       :
//    var       :
//    const     :
// Rckgabe     : boolean; true: Das Objekt hngt an "Verwalter"
// erstellt am  : 26.04.2001
////////////////////////////////////////////////////////////////////////////////
// nderungen:
////////////////////////////////////////////////////////////////////////////////
function TRakEventObject.ListensTo(Verwalter: TRakEventVerwalter): boolean;
begin
  Result := Sender.IndexOf(Verwalter) >= 0;
end;

{ TRakNotifyEvent }

////////////////////////////////////////////////////////////////////////////////
// Klasse       : TRakNotifyEvent
// Methode      : FireEvent
// Sichtbarkeit : public
// Virtual      : ja
// Beschreibung : Wird von einem TRakNotifyEventVerwalter aufgerufen,
//                um das Ereignis aufzurufen.
//                Es wird das Ereignis Event ausgelst.
// Parameter
//    in        : Sender: Das Objekt, welches das Ereignis ausgelst hat
//    out       :
//    var       :
//    const     :
// Rckgabe     :
// erstellt am  : 26.04.2001
////////////////////////////////////////////////////////////////////////////////
// nderungen:
////////////////////////////////////////////////////////////////////////////////
procedure TRakNotifyEvent.FireEvent(Sender: TObject);
begin
  if Assigned(FEvent) then
    FEvent(Sender);
end;

{ TRakNotifyEventVerwalter }

////////////////////////////////////////////////////////////////////////////////
// Klasse       : TRakNotifyEventVerwalter
// Methode      : AddEventObj
// Sichtbarkeit : public
// Virtual      : nein
// Beschreibung : Erzeugt eine neue Instanz von TRakNotifyEvent
//                und gibt sie zurck. Der Aufrufer ist ab dann der
//                Besitzer der Instanz und ist fr die Freigabe verantwortlich
// Parameter
//    in        :
//    out       :
//    var       :
//    const     :
// Rckgabe     : TRakNotifyEvent; Neue Instanz von TRakNotifyEvent
// erstellt am  : 26.04.2001
////////////////////////////////////////////////////////////////////////////////
// nderungen:
////////////////////////////////////////////////////////////////////////////////
function TRakNotifyEventVerwalter.AddEventObj: TRakNotifyEvent;
begin
  Result := TRakNotifyEvent(Add);
end;

////////////////////////////////////////////////////////////////////////////////
// Klasse       : TRakNotifyEventVerwalter
// Konstruktor  : Create
// Sichtbarkeit : public
// Virtual      : nein
// Beschreibung : Hier wird festgelegt, dass TRakNotifyEvent die EventKlasse
//                fr TRakNotifyEventVerwalter ist
// Parameter
//    in        :
//    out       :
//    var       :
//    const     :
// erstellt am  : 26.04.2001
////////////////////////////////////////////////////////////////////////////////
// nderungen:
////////////////////////////////////////////////////////////////////////////////
constructor TRakNotifyEventVerwalter.Create;
begin
  inherited Create(TRakNotifyEvent);
end;

////////////////////////////////////////////////////////////////////////////////
// Klasse       : TRakNotifyEventVerwalter
// Methode      : FireEvent
// Sichtbarkeit : public
// Virtual      : ja
// Beschreibung :  Wird aufgerufen, damit das Ereignis ausgelst wird.
//                Es wird von allen TRakNotifyEvent-Objkten, die sich
//                an den EventVerwalter angehngt haben, deren
//                FireEvent-Methode aufgerufen.
// Parameter
//    in        : Sender: Das Objekt, welches das Ereignis ausgelst hat
//    out       :
//    var       :
//    const     :
// Rckgabe     :
// erstellt am  : 26.04.2001
////////////////////////////////////////////////////////////////////////////////
// nderungen:
////////////////////////////////////////////////////////////////////////////////
procedure TRakNotifyEventVerwalter.FireEvent(Sender: TObject);
var
  i: integer;
begin
  for i := EventList.Count - 1 downto 0 do
  begin
    if i < EventList.Count then
      TRakNotifyEvent(EventList[i]).FireEvent(Sender);
  end;
end;


{ TRakEventItem }

destructor TRakEventItem.Destroy;
begin
  FDestroying := true;
  FEvent.Free;

  inherited;
end;

procedure TRakEventItem.Getrennt(Sender: TObject);
begin
  if not FDestroying then
  begin
    if Event.Count = 0 then
      Free;
  end;
end;

{ TRakEventList }

function TRakEventList.Add(Event: TRakEventObject): TRakEventItem;
begin
  Result := TRakEventItem(BasicAdd(TRakEventItem));
  Result.FEvent := Event;
  Event.FOnGetrennt := Result.Getrennt;
end;

function TRakEventList.Add(Event: TRakEventObject;
  Sender: TObject): TRakEventItem;
begin
  Result := Add(Event);
  Result.Sender := Sender;
  Event.FOnGetrennt := Result.Getrennt;
end;

function TRakEventList.Find(Sender: TObject): TRakEventItem;
var
  i: integer;
begin
  Result := nil;
  for i := 0 to Count - 1 do
  begin
    if Items[i].Sender = Sender then
    begin
      Result := Items[i];
      break;
    end;
  end;
end;

function TRakEventList.FindByVerwalter(
  Verwalter: TRakEventVerwalter): TRakEventItem;
var
  i: integer;
begin
  Result := nil;
  for i := 0 to Count - 1 do
  begin
    if Items[i].Event.ListensTo(Verwalter) then
    begin
      Result := Items[i];
      break;
    end;
  end;
end;

procedure TRakEventList.FreeSenderEvents(Sender: TObject);
var
  i: integer;
begin
  for i := Count - 1 downto 0 do
  begin
    if i < Count then
    begin
      if Items[i].Sender = Sender then
        Items[i].Free;
    end;
  end;
end;

function TRakEventList.GetItems(Idx: integer): TRakEventItem;
begin
  Result := TRakEventItem(inherited Items[Idx]);
end;

end.
